@@ -1,14 +1,14 @@
+import inspect
 import random
 
 from RespawnSimulator.utils import debugOut, echo
 
-MaxBit = 100
-
 
 class _Event:
     # event_id = [0]
 
-    def __init__(self, eid, name, description, conditions, properties, weight=1, tags=[], repeat=False):
+    def __init__(self, eid, name, description, conditions, properties, weight=1, tags=[], repeat=False, rise="{0} +{1}",
+                 decline="{0} -{1}"):
         """
         :param eid: 事件id
         :param name: 事件名
@@ -18,6 +18,9 @@ class _Event:
         :param weight: 权重
         :param tags: 事件的标签
         :param repeat: 是否可以重复发生
+        :param rise: 事件发生，数值上升提示文本，0为属性名，1为变化的数值。e.g. "{0} 上升了{1}点！"
+        :param decline: 事件发生，数值下降提示文本，0为属性名，1为变化的数值。e.g. "{0} 下降了{1}点！"
+        :type eid: int
         :type name: str
         :type description: str
         :type conditions: dict[str,(int,int)]
@@ -25,6 +28,8 @@ class _Event:
         :type weight: int
         :type tags: list[str]
         :type repeat: bool
+        :type rise: str
+        :type decline: str
         """
         self._Id = eid
         self.Name = name
@@ -34,6 +39,8 @@ class _Event:
         self.Conditions = conditions
         self.Properties = properties
         self.Repeat = repeat
+        self.Rise = rise
+        self.Decline = decline
         # self.event_id[0] += 1
         # print(self.event_id[0])
 
@@ -43,6 +50,45 @@ class _Event:
         """
         return self._Id
 
+    def get(self, value_name):
+        """
+        :param value_name: 要获取的事件参数 name,description,conditions,properties等
+        :type value_name: str
+        """
+        vn = value_name[0].upper() + value_name[1:]
+        if vn in self.__dict__:
+            return self.__dict__[vn]
+        else:
+            debugOut("EVENT_GET_ERR", value_name + " doesn't exist.")
+            return None
+
+    def set(self, name=None, description=None, conditions=None, properties=None, weight=None, tags=None,
+            repeat=None, rise=None, decline=None):
+        """
+        :param name: 事件名
+        :param description: 事件描述
+        :param conditions: (>=,<) 如果最小值大于最大值，则数值越大概率越低
+        :param properties: 事件带来的属性影响
+        :param weight: 权重
+        :param tags: 事件的标签
+        :param repeat: 是否可以重复发生
+        :param rise: 事件发生，数值上升提示文本，0为属性名，1为变化的数值。e.g. "{0} 上升了{1}点！"
+        :param decline: 事件发生，数值下降提示文本，0为属性名，1为变化的数值。e.g. "{0} 下降了{1}点！"
+        :type name: str
+        :type description: str
+        :type conditions: dict[str,(int,int)]
+        :type properties: dict[str,int]
+        :type weight: int
+        :type tags: list[str]
+        :type repeat: bool
+        :type rise: str
+        :type decline: str
+        """
+        value_map = locals()
+        for param in inspect.getfullargspec(self.set).args[1:]:
+            if value_map[param] is not None:
+                self.__dict__[param[0].upper() + param[1:]] = value_map[param]
+
     def happen(self, character):
         """
         :param character: 被事件卷入的角色
@@ -61,9 +107,9 @@ class _Event:
                 pass
             else:
                 if value > 0:
-                    echo("{0} +{1}".format(character.Properties[ppt_name].Name, value))
+                    echo(self.Rise.format(character.Properties[ppt_name].Name, value))
                 else:
-                    echo("{0} {1}".format(character.Properties[ppt_name].Name, value))
+                    echo(self.Decline.format(character.Properties[ppt_name].Name, 0 - value))
 
     def cacl_percent(self, character):
         """
@@ -117,11 +163,12 @@ class Events:
         :type empty_event_description: str
         """
         self.Name = name
-        self.events = []
+        self._events = []
         self._total = 0
         self.append(empty_event_name, empty_event_description, {}, {}, 1, [], True)  # 空事件
 
-    def append(self, name, description, conditions, properties, weight=1, tags=[], repeat=False):
+    def append(self, name, description, conditions, properties, weight=1, tags=[], repeat=False, rise="{0} +{1}",
+               decline="{0} -{1}"):
         """
         :param name: 事件名
         :param description: 事件描述
@@ -130,6 +177,8 @@ class Events:
         :param weight: 权重
         :param tags: 事件的标签
         :param repeat: 是否可以重复发生
+        :param rise: 事件发生，数值上升提示文本，0为属性名，1为变化的数值。e.g. "{0} 上升了{1}点！"
+        :param decline: 事件发生，数值下降提示文本，0为属性名，1为变化的数值。e.g. "{0} 下降了{1}点！"
         :type name: str
         :type description: str
         :type conditions: dict[str,(int,int)]
@@ -137,12 +186,15 @@ class Events:
         :type weight: int
         :type tags: list[str]
         :type repeat: bool
+        :type rise: str
+        :type decline: str
         """
-        self.events.append(_Event(self._total, name, description, conditions, properties, weight, tags, repeat))
+        self._events.append(
+            _Event(self._total, name, description, conditions, properties, weight, tags, repeat, rise, decline))
         self._total += 1
 
     def return_events(self):
-        return self.events
+        return self._events
 
     def get_event(self, eid):
         """
@@ -150,7 +202,56 @@ class Events:
         :type eid: int
         :return: _Event
         """
-        return self.events[eid]
+        if eid < len(self._events):
+            return self._events[eid]
+        else:
+            debugOut("Events_Get_Event", "Eid error " + str(eid))
+            return None
+
+    def __add__(self, other):
+        if type(other) == Events:
+            for event in other.return_events()[1:]:
+                self.append(event.Name, event.Description, event.Conditions, event.Properties, event.Weight, event.Tags,
+                            event.Repeat, event.Rise, event.Decline)
+        else:
+            debugOut("EVENTS_ADD_ERR", str(type(other)))
+        return self
+
+    def get(self, eid, value_name):
+        """
+        :param eid: 事件id
+        :param value_name: 要获取的事件参数 name,description,conditions,properties等
+        :type value_name: str
+        :type eid: int
+        """
+        return self.get_event(eid).get(value_name)
+
+    def set(self, eid, name=None, description=None, conditions=None, properties=None, weight=None, tags=None,
+            repeat=None, rise=None, decline=None):
+        """
+        :param eid: 事件id
+        :param name: 事件名
+        :param description: 事件描述
+        :param conditions: (>=,<) 如果最小值大于最大值，则数值越大概率越低
+        :param properties: 事件带来的属性影响
+        :param weight: 权重
+        :param tags: 事件的标签
+        :param repeat: 是否可以重复发生
+        :param rise: 事件发生，数值上升提示文本，0为属性名，1为变化的数值。e.g. "{0} 上升了{1}点！"
+        :param decline: 事件发生，数值下降提示文本，0为属性名，1为变化的数值。e.g. "{0} 下降了{1}点！"
+        :type eid: int
+        :type name: str
+        :type description: str
+        :type conditions: dict[str,(int,int)]
+        :type properties: dict[str,int]
+        :type weight: int
+        :type tags: list[str]
+        :type repeat: bool
+        :type rise: str
+        :type decline: str
+        """
+        self.get_event(eid).set(name, description, conditions, properties, weight, tags,
+                                repeat, rise, decline)
 
     def happen(self, eid, character):
         """
@@ -160,7 +261,7 @@ class Events:
         :type character: RespawnSimulator.character.Character
         :return: None
         """
-        self.events[eid].happen(character)
+        self.get_event(eid).happen(character)
 
     def set_condition(self, eid, ppt_name, section):
         """
@@ -171,8 +272,9 @@ class Events:
         :type section: (int,int)
         :return:
         """
-        if ppt_name in self.events[eid].Conditions:
-            self.events[eid].Conditions[ppt_name] = section
+        ev = self.get_event(eid)
+        if ppt_name in ev.Conditions:
+            ev.Conditions[ppt_name] = section
         else:
             debugOut("Event_Set_Condition", "{0} not found".format(ppt_name))
 
@@ -185,50 +287,50 @@ class Events:
         :type value: int
         :return:
         """
-        if ppt_name in self.events[eid].Properties:
-            self.events[eid].Properties[ppt_name].Value = value
+        ev = self.get_event(eid)
+        if ppt_name in ev.Properties:
+            ev.Properties[ppt_name].Value = value
         else:
             debugOut("Event_Set_Property", "{0} not found".format(ppt_name))
 
+    def god_choose(self, character, density=100) -> int:
+        """
+            :param character: 被决定命运的角色
+            :type character: RespawnSimulator.character.Character
+            :param density: 事件发生密度 数值越大事件越稀疏，即空事件发生概率越大。权重超过此值无意义。
+            :type density: int
+            :return: int 事件id
+            """
 
-def GodChoose(character, events: Events) -> int:
-    """
-    :param character: 被决定命运的角色
-    :type character: RespawnSimulator.character.Character
-    :param events: 事件组
-    :type events: Events
-    :return: int 事件id
-    """
+        def getPercent(elem):
+            return elem[1]
 
-    def getPercent(elem):
-        return elem[1]
+        # events = events.return_events()
+        valid_events = []
+        for event in self._events:
+            percent = event.cacl_percent(character)
+            if percent > 0:  # 排除不可能发生事件
+                valid_events.append((event.eid(), percent, event.Weight))
 
-    events = events.return_events()
-    valid_events = []
-    for event in events:
-        percent = event.cacl_percent(character)
-        if percent > 0:  # 排除不可能发生事件
-            valid_events.append((event.eid(), percent, event.Weight))
+        # print(valid_events)
+        valid_events.sort(key=getPercent, reverse=True)
+        if len(valid_events) <= 0:
+            return 0  # 零号空事件
+        min_percent = valid_events[-1][1]
+        rate = density / min_percent
 
-    # print(valid_events)
-    valid_events.sort(key=getPercent, reverse=True)
-    if len(valid_events) <= 0:
-        return 0  # 零号空事件
-    min_percent = valid_events[-1][1]
-    rate = MaxBit / min_percent
+        in_groove_events = []
+        less_bits = density
+        for item in valid_events:
+            bits = int(item[1] / rate * item[2])
+            in_groove_events.append((item[0], bits))
+            less_bits -= bits
+            if less_bits <= 0:
+                break
+        # print(in_groove_events)
+        n = random.randint(0, density)
+        for item in in_groove_events:
+            if n < item[1]:
+                return item[0]
 
-    in_groove_events = []
-    less_bits = MaxBit
-    for item in valid_events:
-        bits = int(item[1] / rate * item[2])
-        in_groove_events.append((item[0], bits))
-        less_bits -= bits
-        if less_bits <= 0:
-            break
-    # print(in_groove_events)
-    n = random.randint(0, MaxBit)
-    for item in in_groove_events:
-        if n < item[1]:
-            return item[0]
-
-    return 0  # 没有落在事件槽的有效位，空事件
+        return 0
